@@ -34,6 +34,24 @@ import uuid
 
 _request_store = threading.local()
 
+# These arguments will be passed to a new context from the first available
+# header to support backwards compatibility.
+_ENVIRON_HEADERS = {'auth_token': ['HTTP_X_AUTH_TOKEN',
+                                   'HTTP_X_STORAGE_TOKEN'],
+                    'user': ['HTTP_X_USER_ID',
+                             'HTTP_X_USER'],
+                    'tenant': ['HTTP_X_PROJECT_ID',
+                               'HTTP_X_TENANT_ID',
+                               'HTTP_X_TENANT'],
+                    'user_domain': ['HTTP_X_USER_DOMAIN_ID'],
+                    'project_domain': ['HTTP_X_PROJECT_DOMAIN_ID'],
+                    'user_name': ['HTTP_X_USER_NAME'],
+                    'project_name': ['HTTP_X_PROJECT_NAME'],
+                    'user_domain_name': ['HTTP_X_USER_DOMAIN_NAME'],
+                    'project_domain_name': ['HTTP_X_PROJECT_DOMAIN_NAME'],
+                    'request_id': ['openstack.request_id'],
+                    }
+
 
 def generate_request_id():
     """Generate a unique request id."""
@@ -53,19 +71,34 @@ class RequestContext(object):
     def __init__(self, auth_token=None, user=None, tenant=None, domain=None,
                  user_domain=None, project_domain=None, is_admin=False,
                  read_only=False, show_deleted=False, request_id=None,
-                 resource_uuid=None, overwrite=True, roles=None):
+                 resource_uuid=None, overwrite=True, roles=None,
+                 user_name=None, project_name=None, domain_name=None,
+                 user_domain_name=None, project_domain_name=None,
+                 is_admin_project=True):
         """Initialize the RequestContext
 
         :param overwrite: Set to False to ensure that the greenthread local
                           copy of the index is not overwritten.
+        :param is_admin_project: Whether the specified project is specified in
+                                 the token as the admin project. Defaults to
+                                 True for backwards compatibility.
+        :type is_admin_project: bool
         """
         self.auth_token = auth_token
         self.user = user
+        self.user_name = user_name
+        # NOTE (rbradfor):  tenant will become project
+        # See spec discussion on https://review.openstack.org/#/c/290907/
         self.tenant = tenant
+        self.project_name = project_name
         self.domain = domain
+        self.domain_name = domain_name
         self.user_domain = user_domain
+        self.user_domain_name = user_domain_name
         self.project_domain = project_domain
+        self.project_domain_name = project_domain_name
         self.is_admin = is_admin
+        self.is_admin_project = is_admin_project
         self.read_only = read_only
         self.show_deleted = show_deleted
         self.resource_uuid = resource_uuid
@@ -96,7 +129,8 @@ class RequestContext(object):
                 'user_domain_id': self.user_domain,
                 'project_id': self.tenant,
                 'project_domain_id': self.project_domain,
-                'roles': self.roles}
+                'roles': self.roles,
+                'is_admin_project': self.is_admin_project}
 
     def to_dict(self):
         """Return a dictionary of context attributes."""
@@ -119,11 +153,17 @@ class RequestContext(object):
                 'request_id': self.request_id,
                 'resource_uuid': self.resource_uuid,
                 'roles': self.roles,
-                'user_identity': user_idt}
+                'user_identity': user_idt,
+                'is_admin_project': self.is_admin_project}
 
     def get_logging_values(self):
         """Return a dictionary of logging specific context attributes."""
-        values = self.to_dict()
+        values = {'user_name': self.user_name,
+                  'project_name': self.project_name,
+                  'domain_name': self.domain_name,
+                  'user_domain_name': self.user_domain_name,
+                  'project_domain_name': self.project_domain_name}
+        values.update(self.to_dict())
         return values
 
     @classmethod
@@ -148,15 +188,28 @@ class RequestContext(object):
         # Load a new context object from the environment variables set by
         # auth_token middleware. See:
         # http://docs.openstack.org/developer/keystonemiddleware/api/keystonemiddleware.auth_token.html#what-auth-token-adds-to-the-request-for-use-by-the-openstack-service
-        kwargs.setdefault('auth_token', environ.get('HTTP_X_AUTH_TOKEN'))
-        kwargs.setdefault('user', environ.get('HTTP_X_USER_ID'))
-        kwargs.setdefault('tenant', environ.get('HTTP_X_PROJECT_ID'))
-        kwargs.setdefault('user_domain', environ.get('HTTP_X_USER_DOMAIN_ID'))
-        kwargs.setdefault('project_domain',
-                          environ.get('HTTP_X_PROJECT_DOMAIN_ID'))
 
-        roles = environ.get('HTTP_X_ROLES')
-        kwargs.setdefault('roles', roles.split(',') if roles else [])
+        # add kwarg if not specified by user from a list of possible headers
+        for k, v_list in _ENVIRON_HEADERS.items():
+            if k in kwargs:
+                continue
+
+            for v in v_list:
+                if v in environ:
+                    kwargs[k] = environ[v]
+                    break
+
+        if 'roles' not in kwargs:
+            roles = environ.get('HTTP_X_ROLES', environ.get('HTTP_X_ROLE'))
+            roles = [r.strip() for r in roles.split(',')] if roles else []
+            kwargs['roles'] = roles
+
+        if 'is_admin_project' not in kwargs:
+            # NOTE(jamielennox): we default is_admin_project to true because if
+            # nothing is provided we have to assume it is the admin project to
+            # make old policy continue to work.
+            is_admin_proj_str = environ.get('HTTP_X_IS_ADMIN_PROJECT', 'true')
+            kwargs['is_admin_project'] = is_admin_proj_str.lower() == 'true'
 
         return cls(**kwargs)
 
