@@ -14,6 +14,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import hashlib
 import uuid
 
 from oslotest import base as test_base
@@ -22,6 +23,10 @@ from oslo_context import context
 from oslo_context import fixture
 
 
+def generate_id(name):
+    return hashlib.md5(name.encode('utf-8')).hexdigest()
+
+
 class Object(object):
     pass
 
@@ -81,10 +86,15 @@ class ContextTest(test_base.BaseTestCase):
         dct = {
             "auth_token": "token1",
             "user": "user1",
+            "user_name": "user1_name",
             "tenant": "tenant1",
+            "project_name": "tenant1_name",
             "domain": "domain1",
+            "domain_name": "domain1_name",
             "user_domain": "user_domain1",
+            "user_domain_name": "user_domain1_name",
             "project_domain": "project_domain1",
+            "project_domain_name": "project_domain1_name",
             "is_admin": True,
             "read_only": True,
             "show_deleted": True,
@@ -93,17 +103,22 @@ class ContextTest(test_base.BaseTestCase):
             "extra_data": "foo"
         }
         ctx = context.RequestContext.from_dict(dct)
-        self.assertEqual("token1", ctx.auth_token)
-        self.assertEqual("user1", ctx.user)
-        self.assertEqual("tenant1", ctx.tenant)
-        self.assertEqual("domain1", ctx.domain)
-        self.assertEqual("user_domain1", ctx.user_domain)
-        self.assertEqual("project_domain1", ctx.project_domain)
+        self.assertEqual(dct['auth_token'], ctx.auth_token)
+        self.assertEqual(dct['user'], ctx.user)
+        self.assertEqual(dct['tenant'], ctx.tenant)
+        self.assertEqual(dct['domain'], ctx.domain)
+        self.assertEqual(dct['user_domain'], ctx.user_domain)
+        self.assertEqual(dct['project_domain'], ctx.project_domain)
         self.assertTrue(ctx.is_admin)
         self.assertTrue(ctx.read_only)
         self.assertTrue(ctx.show_deleted)
-        self.assertEqual("request1", ctx.request_id)
-        self.assertEqual("instance1", ctx.resource_uuid)
+        self.assertEqual(dct['request_id'], ctx.request_id)
+        self.assertEqual(dct['resource_uuid'], ctx.resource_uuid)
+        self.assertEqual(dct['user_name'], ctx.user_name)
+        self.assertEqual(dct['project_name'], ctx.project_name)
+        self.assertEqual(dct['domain_name'], ctx.domain_name)
+        self.assertEqual(dct['user_domain_name'], ctx.user_domain_name)
+        self.assertEqual(dct['project_domain_name'], ctx.project_domain_name)
 
     def test_from_dict_unknown_keys(self):
         dct = {
@@ -132,27 +147,42 @@ class ContextTest(test_base.BaseTestCase):
 
     def test_from_environ_variables(self):
         auth_token = uuid.uuid4().hex
-        user_id = uuid.uuid4().hex
-        project_id = uuid.uuid4().hex
-        user_domain_id = uuid.uuid4().hex
-        project_domain_id = uuid.uuid4().hex
+        user_name = uuid.uuid4().hex
+        user_id = generate_id(user_name)
+        project_name = uuid.uuid4().hex
+        project_id = generate_id(project_name)
+        user_domain_name = uuid.uuid4().hex
+        user_domain_id = generate_id(user_domain_name)
+        project_domain_name = uuid.uuid4().hex
+        project_domain_id = generate_id(project_domain_name)
         roles = [uuid.uuid4().hex, uuid.uuid4().hex, uuid.uuid4().hex]
+        request_id = uuid.uuid4().hex
 
         environ = {'HTTP_X_AUTH_TOKEN': auth_token,
                    'HTTP_X_USER_ID': user_id,
                    'HTTP_X_PROJECT_ID': project_id,
                    'HTTP_X_USER_DOMAIN_ID': user_domain_id,
                    'HTTP_X_PROJECT_DOMAIN_ID': project_domain_id,
-                   'HTTP_X_ROLES': ','.join(roles)}
+                   'HTTP_X_ROLES': ','.join(roles),
+                   'HTTP_X_USER_NAME': user_name,
+                   'HTTP_X_PROJECT_NAME': project_name,
+                   'HTTP_X_USER_DOMAIN_NAME': user_domain_name,
+                   'HTTP_X_PROJECT_DOMAIN_NAME': project_domain_name,
+                   'openstack.request_id': request_id}
 
         ctx = context.RequestContext.from_environ(environ)
 
         self.assertEqual(auth_token, ctx.auth_token)
         self.assertEqual(user_id, ctx.user)
+        self.assertEqual(user_name, ctx.user_name)
         self.assertEqual(project_id, ctx.tenant)
+        self.assertEqual(project_name, ctx.project_name)
         self.assertEqual(user_domain_id, ctx.user_domain)
+        self.assertEqual(user_domain_name, ctx.user_domain_name)
         self.assertEqual(project_domain_id, ctx.project_domain)
+        self.assertEqual(project_domain_name, ctx.project_domain_name)
         self.assertEqual(roles, ctx.roles)
+        self.assertEqual(request_id, ctx.request_id)
 
     def test_from_environ_no_roles(self):
         ctx = context.RequestContext.from_environ(environ={})
@@ -161,6 +191,75 @@ class ContextTest(test_base.BaseTestCase):
         ctx = context.RequestContext.from_environ(environ={'HTTP_X_ROLES': ''})
         self.assertEqual([], ctx.roles)
 
+    def test_from_environ_deprecated_variables(self):
+        value = uuid.uuid4().hex
+
+        environ = {'HTTP_X_USER': value}
+        ctx = context.RequestContext.from_environ(environ=environ)
+        self.assertEqual(value, ctx.user)
+
+        environ = {'HTTP_X_TENANT_ID': value}
+        ctx = context.RequestContext.from_environ(environ=environ)
+        self.assertEqual(value, ctx.tenant)
+
+        environ = {'HTTP_X_STORAGE_TOKEN': value}
+        ctx = context.RequestContext.from_environ(environ=environ)
+        self.assertEqual(value, ctx.auth_token)
+
+        environ = {'HTTP_X_TENANT': value}
+        ctx = context.RequestContext.from_environ(environ=environ)
+        self.assertEqual(value, ctx.tenant)
+
+        environ = {'HTTP_X_ROLE': value}
+        ctx = context.RequestContext.from_environ(environ=environ)
+        self.assertEqual([value], ctx.roles)
+
+    def test_from_environ_deprecated_precendence(self):
+        old = uuid.uuid4().hex
+        new = uuid.uuid4().hex
+        override = uuid.uuid4().hex
+
+        environ = {'HTTP_X_USER': old,
+                   'HTTP_X_USER_ID': new}
+
+        ctx = context.RequestContext.from_environ(environ=environ)
+        self.assertEqual(ctx.user, new)
+
+        ctx = context.RequestContext.from_environ(environ=environ,
+                                                  user=override)
+        self.assertEqual(ctx.user, override)
+
+        environ = {'HTTP_X_TENANT': old,
+                   'HTTP_X_PROJECT_ID': new}
+
+        ctx = context.RequestContext.from_environ(environ=environ)
+        self.assertEqual(ctx.tenant, new)
+
+        ctx = context.RequestContext.from_environ(environ=environ,
+                                                  tenant=override)
+        self.assertEqual(ctx.tenant, override)
+
+    def test_from_environ_strip_roles(self):
+        environ = {'HTTP_X_ROLES': ' abc\t,\ndef\n,ghi\n\n'}
+        ctx = context.RequestContext.from_environ(environ=environ)
+        self.assertEqual(['abc', 'def', 'ghi'], ctx.roles)
+
+    def test_environ_admin_project(self):
+        environ = {}
+        ctx = context.RequestContext.from_environ(environ=environ)
+        self.assertIs(True, ctx.is_admin_project)
+        self.assertIs(True, ctx.to_policy_values()['is_admin_project'])
+
+        environ = {'HTTP_X_IS_ADMIN_PROJECT': 'True'}
+        ctx = context.RequestContext.from_environ(environ=environ)
+        self.assertIs(True, ctx.is_admin_project)
+        self.assertIs(True, ctx.to_policy_values()['is_admin_project'])
+
+        environ = {'HTTP_X_IS_ADMIN_PROJECT': 'False'}
+        ctx = context.RequestContext.from_environ(environ=environ)
+        self.assertIs(False, ctx.is_admin_project)
+        self.assertIs(False, ctx.to_policy_values()['is_admin_project'])
+
     def test_from_function_and_args(self):
         ctx = context.RequestContext(user="user1")
         arg = []
@@ -179,11 +278,16 @@ class ContextTest(test_base.BaseTestCase):
     def test_values(self):
         auth_token = "token1"
         # test unicode support
-        user = u"John Gāo"
-        tenant = "tenant1"
-        domain = "domain1"
-        user_domain = "user_domain1"
-        project_domain = "project_domain1"
+        user_name = u"John Gāo"
+        user_id = generate_id(user_name)
+        project_name = 'tenant1'
+        project_id = generate_id(project_name)
+        domain_name = 'domain1'
+        domain_id = generate_id(domain_name)
+        user_domain_name = 'user_domain1'
+        user_domain_id = generate_id(user_domain_name)
+        project_domain_name = 'project_domain1'
+        project_domain_id = generate_id(project_domain_name)
         is_admin = True
         read_only = True
         show_deleted = True
@@ -191,22 +295,32 @@ class ContextTest(test_base.BaseTestCase):
         resource_uuid = "uuid1"
 
         ctx = context.RequestContext(auth_token=auth_token,
-                                     user=user,
-                                     tenant=tenant,
-                                     domain=domain,
-                                     user_domain=user_domain,
-                                     project_domain=project_domain,
+                                     user=user_id,
+                                     user_name=user_name,
+                                     tenant=project_id,
+                                     project_name=project_name,
+                                     domain=domain_id,
+                                     domain_name=domain_name,
+                                     user_domain=user_domain_id,
+                                     user_domain_name=user_domain_name,
+                                     project_domain=project_domain_id,
+                                     project_domain_name=project_domain_name,
                                      is_admin=is_admin,
                                      read_only=read_only,
                                      show_deleted=show_deleted,
                                      request_id=request_id,
                                      resource_uuid=resource_uuid)
         self.assertEqual(auth_token, ctx.auth_token)
-        self.assertEqual(user, ctx.user)
-        self.assertEqual(tenant, ctx.tenant)
-        self.assertEqual(domain, ctx.domain)
-        self.assertEqual(user_domain, ctx.user_domain)
-        self.assertEqual(project_domain, ctx.project_domain)
+        self.assertEqual(user_id, ctx.user)
+        self.assertEqual(user_name, ctx.user_name)
+        self.assertEqual(project_id, ctx.tenant)
+        self.assertEqual(project_name, ctx.project_name)
+        self.assertEqual(domain_id, ctx.domain)
+        self.assertEqual(domain_name, ctx.domain_name)
+        self.assertEqual(user_domain_id, ctx.user_domain)
+        self.assertEqual(user_domain_name, ctx.user_domain_name)
+        self.assertEqual(project_domain_id, ctx.project_domain)
+        self.assertEqual(project_domain_name, ctx.project_domain_name)
         self.assertEqual(is_admin, ctx.is_admin)
         self.assertEqual(read_only, ctx.read_only)
         self.assertEqual(show_deleted, ctx.show_deleted)
@@ -227,58 +341,27 @@ class ContextTest(test_base.BaseTestCase):
         self.assertIn('resource_uuid', d)
         self.assertIn('user_identity', d)
         self.assertIn('roles', d)
+        self.assertNotIn('user_name', d)
+        self.assertNotIn('project_name', d)
+        self.assertNotIn('domain_name', d)
+        self.assertNotIn('user_domain_name', d)
+        self.assertNotIn('project_domain_name', d)
 
         self.assertEqual(auth_token, d['auth_token'])
-        self.assertEqual(tenant, d['tenant'])
-        self.assertEqual(domain, d['domain'])
-        self.assertEqual(user_domain, d['user_domain'])
-        self.assertEqual(project_domain, d['project_domain'])
+        self.assertEqual(project_id, d['tenant'])
+        self.assertEqual(domain_id, d['domain'])
+        self.assertEqual(user_domain_id, d['user_domain'])
+        self.assertEqual(project_domain_id, d['project_domain'])
         self.assertEqual(is_admin, d['is_admin'])
         self.assertEqual(read_only, d['read_only'])
         self.assertEqual(show_deleted, d['show_deleted'])
         self.assertEqual(request_id, d['request_id'])
         self.assertEqual(resource_uuid, d['resource_uuid'])
-        user_identity = "%s %s %s %s %s" % (user, tenant, domain,
-                                            user_domain, project_domain)
+        user_identity = "%s %s %s %s %s" % (user_id, project_id, domain_id,
+                                            user_domain_id, project_domain_id)
         self.assertEqual(user_identity, d['user_identity'])
         self.assertEqual([], d['roles'])
 
-    def test_get_logging_values(self):
-        auth_token = "token1"
-        user = "user1"
-        tenant = "tenant1"
-        domain = "domain1"
-        user_domain = "user_domain1"
-        project_domain = "project_domain1"
-        is_admin = True
-        read_only = True
-        show_deleted = True
-        request_id = "id1"
-        resource_uuid = "uuid1"
-
-        ctx = context.RequestContext(auth_token=auth_token,
-                                     user=user,
-                                     tenant=tenant,
-                                     domain=domain,
-                                     user_domain=user_domain,
-                                     project_domain=project_domain,
-                                     is_admin=is_admin,
-                                     read_only=read_only,
-                                     show_deleted=show_deleted,
-                                     request_id=request_id,
-                                     resource_uuid=resource_uuid)
-        self.assertEqual(auth_token, ctx.auth_token)
-        self.assertEqual(user, ctx.user)
-        self.assertEqual(tenant, ctx.tenant)
-        self.assertEqual(domain, ctx.domain)
-        self.assertEqual(user_domain, ctx.user_domain)
-        self.assertEqual(project_domain, ctx.project_domain)
-        self.assertEqual(is_admin, ctx.is_admin)
-        self.assertEqual(read_only, ctx.read_only)
-        self.assertEqual(show_deleted, ctx.show_deleted)
-        self.assertEqual(request_id, ctx.request_id)
-        self.assertEqual(resource_uuid, ctx.resource_uuid)
-
         d = ctx.get_logging_values()
         self.assertIn('auth_token', d)
         self.assertIn('user', d)
@@ -292,20 +375,18 @@ class ContextTest(test_base.BaseTestCase):
         self.assertIn('request_id', d)
         self.assertIn('resource_uuid', d)
         self.assertIn('user_identity', d)
+        self.assertIn('roles', d)
+        self.assertIn('user_name', d)
+        self.assertIn('project_name', d)
+        self.assertIn('domain_name', d)
+        self.assertIn('user_domain_name', d)
+        self.assertIn('project_domain_name', d)
 
-        self.assertEqual(auth_token, d['auth_token'])
-        self.assertEqual(tenant, d['tenant'])
-        self.assertEqual(domain, d['domain'])
-        self.assertEqual(user_domain, d['user_domain'])
-        self.assertEqual(project_domain, d['project_domain'])
-        self.assertEqual(is_admin, d['is_admin'])
-        self.assertEqual(read_only, d['read_only'])
-        self.assertEqual(show_deleted, d['show_deleted'])
-        self.assertEqual(request_id, d['request_id'])
-        self.assertEqual(resource_uuid, d['resource_uuid'])
-        user_identity = "%s %s %s %s %s" % (user, tenant, domain,
-                                            user_domain, project_domain)
-        self.assertEqual(user_identity, d['user_identity'])
+        self.assertEqual(user_name, d['user_name'])
+        self.assertEqual(project_name, d['project_name'])
+        self.assertEqual(domain_name, d['domain_name'])
+        self.assertEqual(user_domain_name, d['user_domain_name'])
+        self.assertEqual(project_domain_name, d['project_domain_name'])
 
     def test_dict_empty_user_identity(self):
         ctx = context.RequestContext()
@@ -328,6 +409,7 @@ class ContextTest(test_base.BaseTestCase):
         project_domain = uuid.uuid4().hex
         roles = [uuid.uuid4().hex, uuid.uuid4().hex, uuid.uuid4().hex]
 
+        # default is_admin_project is True
         ctx = context.RequestContext(user=user,
                                      user_domain=user_domain,
                                      tenant=tenant,
@@ -338,4 +420,22 @@ class ContextTest(test_base.BaseTestCase):
                           'user_domain_id': user_domain,
                           'project_id': tenant,
                           'project_domain_id': project_domain,
-                          'roles': roles}, ctx.to_policy_values())
+                          'roles': roles,
+                          'is_admin_project': True},
+                         ctx.to_policy_values())
+
+        # is_admin_project False gets passed through
+        ctx = context.RequestContext(user=user,
+                                     user_domain=user_domain,
+                                     tenant=tenant,
+                                     project_domain=project_domain,
+                                     roles=roles,
+                                     is_admin_project=False)
+
+        self.assertEqual({'user_id': user,
+                          'user_domain_id': user_domain,
+                          'project_id': tenant,
+                          'project_domain_id': project_domain,
+                          'roles': roles,
+                          'is_admin_project': False},
+                         ctx.to_policy_values())
